@@ -4,6 +4,7 @@ import subprocess
 import zipfile
 import sys
 import getopt
+import shutil
 
 """
 Script will create an AWS Lambda function deployment.
@@ -32,9 +33,27 @@ root_project_dir = ''
 # file will be included in the deployment.
 deployment_files = []
 
+default_requirements_file_name = "requirements.txt"
+
+def _read_test_requirements():
+    filename = os.path.join(root_project_dir, "requirements-test.txt")
+    if not os.path.exists(filename):
+        return None
+
+    with open(filename, 'r') as f:
+        install_requirements = f.readlines()
+
+    return install_requirements
+
 
 def _read_requirements():
-    with open("{0}/requirements.txt".format(root_project_dir), 'r') as f:
+    filename = os.path.join(root_project_dir, default_requirements_file_name)
+    if not os.path.exists(filename):
+        print("WARNING: A 'requirements.txt' file was not found and at a minimum it is expected that pyalexa-skill and its dependents would be in there.")
+        print("run:  pip freeze > requirements.txt   to generate this file.")
+        return None
+
+    with open(filename, 'r') as f:
         install_requirements = f.readlines()
 
     return install_requirements
@@ -56,7 +75,7 @@ def _make_deployment_dir():
     max_deployment_number = -1
     for deployment_dir in all_deployment_directories:
         dir_name_elements = deployment_dir.split("_")
-        if (len(dir_name_elements) == 2):
+        if len(dir_name_elements) == 2:
             if int(dir_name_elements[1]) > max_deployment_number:
                 max_deployment_number = int(dir_name_elements[1])
 
@@ -64,13 +83,25 @@ def _make_deployment_dir():
         max_deployment_number = 0
 
     deployment_name = "deployment_{0}".format(max_deployment_number + 1)
-    new_deployment_dir_path = "{0}/{1}".format(root_deployments_dir, deployment_name)
+    new_deployment_dir_path = os.path.join(root_deployments_dir, deployment_name)
 
     if not os.path.exists(new_deployment_dir_path):
         os.mkdir(new_deployment_dir_path)
 
     return (new_deployment_dir_path, deployment_name)
 
+def _install_test_requirements(deployment_requirements, deployment_dir):
+    """
+    pip install -i https://testpypi.python.org/pypi <requirements line> -t <deployment_dir>
+    :param deployment_requirements
+    :param deployment_dir:
+    :return:
+    """
+    if os.path.exists(deployment_dir):
+        for requirement in deployment_requirements:
+            if not requirement.startswith('#'):
+                cmd = "pip install -i https://testpypi.python.org/pypi {0} -t {1}".format(requirement, deployment_dir).split()
+                return_code = subprocess.call(cmd, shell=False)
 
 def _install_requirements(deployment_requirements, deployment_dir):
     """
@@ -86,13 +117,14 @@ def _install_requirements(deployment_requirements, deployment_dir):
                 return_code = subprocess.call(cmd, shell=False)
 
 
-def _copy_deployment_files(deployment_dir):
-    for deployment_file in deployment_files:
-        if os.path.exists(deployment_file):
-            cmd = "cp {0} {1}".format(deployment_file, deployment_dir).split()
-            return_code = subprocess.call(cmd, shell=False)
+def _copy_deployment_files(deployment_data):
+    for item in deployment_data:
+        if os.path.exists(item['from']):
+            shutil.copy2(item['from'], item['to'])
+            # cmd = "cp {0} {1}".format(item['from'], item['to']).split()
+            # return_code = subprocess.call(cmd, shell=False)
         else:
-            raise NameError("Deployment file not found [{0}]".format(deployment_file))
+            raise NameError("Deployment file not found [{0}]".format(item['from']))
 
 
 def zipdir(dirPath=None, zipFilePath=None, includeDirInZip=False):
@@ -158,46 +190,99 @@ def zipdir(dirPath=None, zipFilePath=None, includeDirInZip=False):
     outFile.close()
 
 
+def make_target_dirs(target_paths):
+    for dirname in set(os.path.dirname(p) for p in target_paths):
+        if not os.path.isdir(dirname):
+            os.makedirs(dirname)
+
+
 def main(argv):
-    global root_deployments_dir, root_project_dir
+    global root_deployments_dir, root_project_dir, default_requirements_file_name
     include_files = ''
 
     try:
-        opts, args = getopt.getopt(argv, "hr:i:", ["root=", "include="])
+        opts, args = getopt.getopt(argv, "hr:i:l:", ["root=", "include=", "libraries"])
     except getopt.GetoptError:
-        print 'create_aws_lambda.py -r <root project dir> -i <include files>'
-        print 'if -r option not supplied it will look for PWD environment variable'
+        print('create_aws_lambda.py -r <root project dir> -i <include files> -l <file of python libraries, e.g. requirements.txt>')
+        print('if -r option not supplied it will look for PWD environment variable')
+        print('if -l option is not supplied, the default library file will be requirements.txt')
         sys.exit(2)
     for opt, arg in opts:
         if opt == '-h':
-            print 'create_aws_lambda.py -r <root project dir> -i <include files>'
-            print 'if -r option not supplied it will look for PWD environment variable'
-            print '<include files> are relative to root project dir'
+            print('create_aws_lambda.py -r <root project dir> -i <include files>')
+            print('if -r option not supplied it will look for PWD environment variable')
+            print('<include files> are relative to root project dir')
             sys.exit()
         elif opt in ("-r", "--root"):
             root_project_dir = arg
         elif opt in ("-i", "--include"):
-            include_files = arg
+            # incase the options are surrounded by quotes
+            include_files = arg.replace("'",'')
+        elif opt in ("-l", "--libraries"):
+            default_requirements_file_name = arg
 
     if not root_project_dir:
         root_project_dir = os.environ.get("PWD")
         if root_project_dir is None:
-            raise ValueError("Must supply -r or --root option")
+            root_project_dir = os.getcwd()
+            if root_project_dir is None:
+                raise ValueError("Must supply -r or --root option")
 
     if not include_files:
         raise ValueError("Must supply -i or --include option")
 
-    for include_file in include_files.split(","):
-        deployment_files.append("{0}/{1}".format(root_project_dir,include_file.strip()))
+    print(f"Using Library file: {default_requirements_file_name}")
 
-    root_deployments_dir = "{0}/deployments".format(root_project_dir)
+    root_deployments_dir = os.path.join(root_project_dir, 'deployments')   #"{0}/deployments".format(root_project_dir)
     (deployment_dir, deployment_name) = _make_deployment_dir()
-    _copy_deployment_files(deployment_dir)
+
+    for include_file in include_files.split(","):
+        d = {
+            "from": os.path.join(root_project_dir, include_file.strip()),
+            "to": os.path.join(deployment_dir, include_file.strip())
+        }
+        make_target_dirs([d['to']])
+        deployment_files.append(d)
+
+    _copy_deployment_files(deployment_files)
+
+    # standard requirements file
     install_requirements = _read_requirements()
-    _install_requirements(install_requirements, deployment_dir)
+    if install_requirements:
+        _install_requirements(install_requirements, deployment_dir)
 
-    zipdir(deployment_dir, "{0}/{1}.zip".format(root_deployments_dir, deployment_name))
+    # test requirements file (requirements-test.txt)
+    install_requirements = _read_test_requirements()
+    if install_requirements:
+        _install_test_requirements(install_requirements, deployment_dir)
 
 
+    deployment_num_file_name = os.path.join(root_deployments_dir, '.deployment_number.txt') #"{0}/.deployment_number.txt".format(root_deployments_dir)
+    if not os.path.isfile(deployment_num_file_name):
+        # first time - be safe and start at deployment 100
+        with open(deployment_num_file_name, 'w') as f:
+            f.write("100\n")
+
+    next_deployment_number = None
+    with open(deployment_num_file_name, 'r+') as f:
+        line1 = f.readline()
+        try:
+            next_deployment_number = int(line1.strip()) + 1
+        except:
+            next_deployment_number = 1
+
+        deployment_dir_path = os.path.join(root_deployments_dir, "deployment_{0}".format(next_deployment_number))
+        deployment_zip_path = os.path.join(root_deployments_dir, "deployment_{0}.zip".format(next_deployment_number))
+        while os.path.exists(deployment_dir_path) or os.path.exists(deployment_zip_path):
+            next_deployment_number = next_deployment_number + 1
+
+        f.seek(0)
+        f.write("{0}\n".format(next_deployment_number))
+
+    deployment_zip_filename = os.path.join(root_deployments_dir, "deployment_{0}.zip".format(next_deployment_number))
+    zipdir(deployment_dir, deployment_zip_filename)
+    shutil.rmtree(deployment_dir, ignore_errors=True)
+    print("Created deployment zip file: {0}".format(deployment_zip_filename))
+
 if __name__ == "__main__":
     main(sys.argv[1:])
